# -*-coding:utf-8 -*-
"""
:创建时间: 2022/8/4 16:45
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

3d向量计算图模块
"""
from __future__ import unicode_literals, print_function, division
import cpmel.cmds as cc
from rig_operation_graph._utils import *


def vector_add(ctx, a, b):
    n = ctx.create_node('plusMinusAverage')
    n.rename('vector_add')
    n['operation'] = 1
    set_or_connect_3d(a, n.input3D[0])
    set_or_connect_3d(b, n.input3D[1])
    return n.output3D


add3d = vector_add


def vector_sub(ctx, a, b):
    n = ctx.create_node('plusMinusAverage')
    n.rename('vector_sub')
    n['operation'] = 2
    set_or_connect_3d(a, n.input3D[0])
    set_or_connect_3d(b, n.input3D[1])
    return n.output3D


sub3d = vector_sub


def vector_mul(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('vector_mul')
    n['operation'] = 1
    set_or_connect_3d(a, n.input1)
    set_or_connect_3d(b, n.input2)
    return n.output


mul3d = vector_mul


def vector_div(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('vector_div')
    n['operation'] = 2
    set_or_connect_3d(a, n.input1)
    set_or_connect_3d(b, n.input2)
    return n.output


div3d = vector_div


def vector_pow(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('vector_pow')
    n['operation'] = 3
    set_or_connect_3d(a, n.input1)
    set_or_connect_3d(b, n.input2)
    return n.output


pow3d = vector_pow


def vector_mean(ctx, *values):
    n = ctx.create_node('plusMinusAverage')
    n.rename('vector_mean')
    n['operation'] = 3
    for id_, v in enumerate(values):
        set_or_connect_3d(v, n.input3D[id_])
    return n.output3D


mean3d = vector_mean


def vector_sum(ctx, *values):
    n = ctx.create_node('plusMinusAverage')
    n.rename('vector_sum')
    n['operation'] = 1
    for id_, v in enumerate(values):
        set_or_connect_3d(v, n.input3D[id_])
    return n.output3D


sum3d = vector_sum


def vector_dot_product(ctx, left, right):
    n = ctx.create_node('vectorProduct')
    n.rename('vector_dot_product')
    n['operation'] = 1
    n['normalizeOutput'] = False

    set_or_connect_3d(left, n.attr('input1'))
    set_or_connect_3d(right, n.attr('input2'))
    return n.attr('outputX')


def vector_cross_product(ctx, left, right):
    n = ctx.create_node('vectorProduct')
    n.rename('vector_cross_product')
    n['operation'] = 2
    n['normalizeOutput'] = False

    set_or_connect_3d(left, n.attr('input1'))
    set_or_connect_3d(right, n.attr('input2'))
    return n.attr('output')


def vector_normalization(ctx, in_attr):
    if is_attr(in_attr):
        in_attr = cc.new_object(in_attr)
        if in_attr.type() == 'float3':
            n = in_attr.node()
            if cc.objectType(n) == 'vectorProduct':
                n['normalizeOutput'] = True
                return in_attr
    n = ctx.create_node('vectorProduct')
    n.rename('vector_normalization')
    n['operation'] = 0
    n['normalizeOutput'] = True

    set_or_connect_3d(in_attr, n.attr('input1'))
    return n.attr('output')


__all__ = [
    'add3d', 'sub3d', 'mul3d', 'div3d', 'pow3d', 'mean3d', 'sum3d',
    'vector_add', 'vector_sub', 'vector_mul', 'vector_div', 'vector_pow', 'vector_mean', 'vector_sum',
    'vector_dot_product', 'vector_cross_product', 'vector_normalization',
]

if __name__ == "__main__":
    from rig_core.ctx import Ctx
    from maya_test_tools import question_open_maya_gui

    ctx = Ctx()

    out_attr = vector_add(ctx, (1, 1, 1), (-1, -1, -1))
    print(out_attr, out_attr.get_value())

    out_attr = vector_sub(ctx, out_attr, (-1, 1, -1))
    print(out_attr, out_attr.get_value())

    out_attr = vector_mul(ctx, out_attr, (1.5, 1.5, 1.5))
    print(out_attr, out_attr.get_value())

    out_attr = vector_div(ctx, out_attr, (3, 3, 3))
    print(out_attr, out_attr.get_value())

    out_attr = vector_pow(ctx, out_attr, (3, 3, 3))
    print(out_attr, out_attr.get_value())

    out_attr = vector_mean(ctx, out_attr, (0, 0, 0), (0, 0, 0), (0, 0, 0))
    print(out_attr, out_attr.get_value())

    out_attr = vector_sum(ctx, out_attr, out_attr, out_attr, out_attr)
    print(out_attr, out_attr.get_value())

    out_attr = vector_dot_product(ctx, (1, 0, 0), (1, 0, 0))
    print(out_attr, out_attr.get_value())
    assert 0.999 < out_attr.get_value() < 1.0001

    out_attr = vector_cross_product(ctx, (0, 0, 1), (1, 0, 0))
    print(out_attr, out_attr.get_value())
    assert -0.0001 < out_attr.get_value().x < 0.0001
    assert 0.999 < out_attr.get_value().y < 1.0001
    assert -0.0001 < out_attr.get_value().z < 0.0001

    out_attr = vector_normalization(ctx, (0, 0.5, 0))
    print(out_attr, out_attr.get_value())
    assert -0.0001 < out_attr.get_value().x < 0.0001
    assert 0.999 < out_attr.get_value().y < 1.0001
    assert -0.0001 < out_attr.get_value().z < 0.0001

    out_attr = vector_cross_product(ctx, (0, 0, 5), (5, 0, 0))
    print(out_attr, out_attr.get_value())
    new_out_attr = vector_normalization(ctx, out_attr)
    print(new_out_attr, new_out_attr.get_value())
    assert new_out_attr == out_attr

    question_open_maya_gui()



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/29 6:50
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function, division

import sys

import cpmel.cmds as cc

unicode_t = type('')
bytes_t = type(b'')

if sys.version_info.major < 3:
    num_t = (int, long, float)
else:
    num_t = (int, float)


def is_attr(attr):
    return isinstance(attr, (unicode_t, bytes_t, cc.Attr))


def set_or_connect(in_value, out_attr):
    if is_attr(in_value):
        in_attr = cc.new_object(in_value)
        in_attr >> out_attr
        return
    out_attr.set_value(in_value)


def set_or_connect_3d(in_value, out_attr):
    if is_attr(in_value):
        in_attr = cc.new_object(in_value)
        in_attr >> out_attr
        return

    attr_plug = out_attr.api1_m_plug()
    out_attr_list = [cc.new_object(attr_plug.child(i).name()) for i in range(attr_plug.numChildren())]

    set_or_connect(in_value[0], out_attr_list[0])
    set_or_connect(in_value[1], out_attr_list[1])
    set_or_connect(in_value[2], out_attr_list[2])


def set_or_connect_matrix(ctx, in_value, out_attr):
    if is_attr(in_value):
        in_attr = cc.new_object(in_value)
        in_attr >> out_attr
        return
    if isinstance(in_value, (list, tuple)):
        if (len(in_value) == 4 or len(in_value) == 16):
            opened_in_value = [t for i in in_value for t in (i if isinstance(i, (list, tuple)) else [i])]
            if len(opened_in_value) == 16:
                n = ctx.create_node('fourByFourMatrix')
                n.rename('convert_to_matrix')
                for o, i in zip(opened_in_value, ['in00', 'in01', 'in02', 'in03',
                                                  'in10', 'in11', 'in12', 'in13',
                                                  'in20', 'in21', 'in22', 'in23',
                                                  'in30', 'in31', 'in32', 'in33', ]):
                    set_or_connect(o, n.attr(i))
                in_attr = n.attr('output')
                in_attr >> out_attr
                return

    out_attr.set_value(in_value)


def set_or_connect_quaternion(in_value, out_attr):
    if is_attr(in_value):
        in_attr = cc.new_object(in_value)
        in_attr >> out_attr
        return
    attr_plug = out_attr.api1_m_plug()
    out_attr_list = [cc.new_object(attr_plug.child(i).name()) for i in range(attr_plug.numChildren())]

    set_or_connect(in_value[0], out_attr_list[0])
    set_or_connect(in_value[1], out_attr_list[1])
    set_or_connect(in_value[2], out_attr_list[2])
    set_or_connect(in_value[3], out_attr_list[3])


__all__ = ['is_attr', 'set_or_connect', 'set_or_connect_3d', 'set_or_connect_matrix', 'set_or_connect_quaternion']



#!/usr/bin/python
# -*-coding:utf-8 -*-
from __future__ import unicode_literals, print_function
import setuptools

lib_name = 'maya-rig-operation-graph'

author = 'cpcgskill',
author_email = 'cpcgskill@outlook.com'

version = '0.4.3'

description = 'maya计算图的实现'
with open("README.rst", "rb") as f:
    long_description = f.read().decode(encoding='utf-8')

project_homepage = 'https://github.com/cpcgskill/maya-rig-operation-graph'
project_urls = {
    'Bug Tracker': 'https://github.com/cpcgskill/maya-rig-operation-graph/issues',
}
license = 'Apache Software License (Apache 2.0)'

packages = ['rig_operation_graph']
python_requires = '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*'
install_requires = [
    'maya-rig-core>=0.2',
]

setuptools.setup(
    name=lib_name,
    version=version,
    author=author,
    author_email=author_email,
    description=description,
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url=project_homepage,
    project_urls=project_urls,
    license=license,
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    package_dir={"": "src"},
    packages=packages,
    package_data={'rig_operation_graph': ['*.pyi']},
    python_requires=python_requires,
    # 指定依赖
    install_requires=install_requires,
)



# -*-coding:utf-8 -*-
"""
:创建时间: 2022/10/6 10:58
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

"""
from __future__ import unicode_literals, print_function, division
import cpmel.cmds as cc
from rig_core.all import *

if False:
    from typing import AnyStr, Any, List, Tuple, Dict

__all__ = ['curve_length']


def curve_length(ctx, curve):
    """
    获得曲线长度

    :type ctx: Ctx
    :type curve: cc.Transform or cc.Curve
    :rtype: cc.Attr
    """
    curve_info = cc.arclen(curve, ch=True, n='curve_length')
    ctx.add_nodes(curve_info)
    return curve_info.attr('arcLength')


if __name__ == '__main__':
    def test():
        from maya_test_tools import open_file, question_open_maya_gui

        cc.eval(
            '''curve -d 3 -p -8.392285 0 -3.922698 -p -7.125865 0 -0.366994 -p -4.593023 0 6.744413 -p 6.853808 0 8.101182 -p -3.133833 0 -3.155691 -p 0.739452 0 -4.285575 -p 5.09475 0 -1.164347 -p 7.272399 0 0.396268 -k 0 -k 0 -k 0 -k 1 -k 2 -k 3 -k 4 -k 5 -k 5 -k 5 ;''')

        ctx = Ctx()

        print('curve_length', curve_length(ctx, cc.new_object('curve1')).value)

        question_open_maya_gui()


    test()
